var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { ChangeDetectorRef, ComponentFactoryResolver, Directive, ElementRef, EventEmitter, forwardRef, HostListener, Injector, Input, Output, Renderer2, TemplateRef, ViewContainerRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { fromEvent } from 'rxjs/observable/fromEvent';
import { of } from 'rxjs/observable/of';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/filter';
import 'rxjs/add/operator/debounceTime';
import 'rxjs/add/operator/distinctUntilChanged';
import 'rxjs/add/operator/switchMap';
import 'rxjs/add/operator/do';
import { HiNGConfig } from '../hi.config';
import { PositionService } from '../position/positioning.service';
import { AutoCompletePopupComponent } from './auto-complete-popup.component';
import { stopPropagationIfExist } from '../utils/dom-utils';
var AutoCompleteDirective = (function () {
    function AutoCompleteDirective(elementRef, viewContainerRef, componentFactoryResolver, renderer, injector, positionService, hiNGConfig, changeDetectorRef) {
        this.elementRef = elementRef;
        this.viewContainerRef = viewContainerRef;
        this.componentFactoryResolver = componentFactoryResolver;
        this.renderer = renderer;
        this.injector = injector;
        this.positionService = positionService;
        this.hiNGConfig = hiNGConfig;
        this.changeDetectorRef = changeDetectorRef;
        this.appendBody = false;
        this.selectValueChange = new EventEmitter();
        this.placement = 'bottom-left';
        this.onChange = function (_) { return null; };
        this.onTouched = function () { return null; };
        this.valueChanges = this.registerInputEvent(elementRef);
        this.delay = hiNGConfig.autoComplete.delay;
        this.minLength = hiNGConfig.autoComplete.minLength;
        this.itemTemplate = hiNGConfig.autoComplete.itemTemplate;
        this.noResultItemTemplate = hiNGConfig.autoComplete.noResultItemTemplate;
        this.formatter = hiNGConfig.autoComplete.formatter;
        this.valueParser = hiNGConfig.autoComplete.valueParser;
    }
    AutoCompleteDirective_1 = AutoCompleteDirective;
    AutoCompleteDirective.prototype.ngOnInit = function () {
        var _this = this;
        this.subscription = this.valueChanges
            .subscribe(function (source) { return _this.onSourceChange(source); });
        var factory = this.componentFactoryResolver.resolveComponentFactory(AutoCompletePopupComponent);
        setTimeout(function () {
            _this.popupRef = _this.viewContainerRef.createComponent(factory, _this.viewContainerRef.length, _this.injector);
            _this.fillPopup();
            _this.positionPopup();
            _this.popupRef.instance.registerOnChange(function (item) {
                var value = _this.valueParser(item);
                _this.writeValue(value);
                _this.onChange(value);
                _this.hidePopup();
                _this.selectValueChange.emit(item);
            });
        }, 0);
    };
    Object.defineProperty(AutoCompleteDirective.prototype, "dataSource", {
        set: function (dataSource) {
            if (dataSource) {
                this.arraySource = dataSource;
                this.setupArraySource();
            }
        },
        enumerable: true,
        configurable: true
    });
    AutoCompleteDirective.prototype.setupArraySource = function () {
        var _this = this;
        if (this.popupRef) {
            this.popupRef.instance.source = this.arraySource;
        }
        this.onSearch = function (term) {
            return of(_this.arraySource
                .filter(function (item) { return _this.formatter(item).toLowerCase().indexOf((term || '').toLowerCase()) !== -1; }));
        };
    };
    AutoCompleteDirective.prototype.writeValue = function (obj) {
        this.value = obj || '';
        this.writeInputValue(this.value);
    };
    AutoCompleteDirective.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    AutoCompleteDirective.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    AutoCompleteDirective.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
        this.renderer.setProperty(this.elementRef.nativeElement, 'disabled', isDisabled);
        if (this.popupRef) {
            this.popupRef.instance.setDisabledState(isDisabled);
        }
    };
    AutoCompleteDirective.prototype.ngOnDestroy = function () {
        this.unSubscription();
        this.removePopView();
    };
    AutoCompleteDirective.prototype.removePopView = function () {
        if (this.popupRef) {
            this.popupRef.destroy();
        }
    };
    AutoCompleteDirective.prototype.onBlur = function () {
        this.onTouched();
    };
    AutoCompleteDirective.prototype.onEscKeyup = function ($event) {
        this.hidePopup();
    };
    AutoCompleteDirective.prototype.onEnterKeyDown = function ($event) {
        if (this.popupRef && this.popupRef.instance.isOpen) {
            $event.preventDefault();
            $event.stopPropagation();
            this.popupRef.instance.selectCurrentItem();
            this.hidePopup();
        }
    };
    AutoCompleteDirective.prototype.onArrowUpKeyDown = function ($event) {
        if (this.popupRef) {
            $event.preventDefault();
            $event.stopPropagation();
            this.popupRef.instance.prev();
        }
    };
    AutoCompleteDirective.prototype.onArrowDownKeyDown = function ($event) {
        if (this.popupRef) {
            $event.preventDefault();
            $event.stopPropagation();
            this.popupRef.instance.next();
        }
    };
    AutoCompleteDirective.prototype.onDocumentClick = function ($event) {
        var hostElement = this.elementRef.nativeElement;
        if ($event.target !== hostElement) {
            this.hidePopup();
        }
    };
    AutoCompleteDirective.prototype.onSourceChange = function (source, term) {
        if (term !== undefined) {
            this.value = term;
        }
        var pop = this.popupRef.instance;
        pop.reset();
        this.fillPopup(source, this.value);
        if ((source && source.length) || this.noResultItemTemplate) {
            pop.show();
            this.positionPopup();
            this.changeDetectorRef.markForCheck();
        }
    };
    AutoCompleteDirective.prototype.hidePopup = function () {
        if (this.popupRef) {
            this.popupRef.instance.activeIndex = 0;
            this.popupRef.instance.hide();
        }
    };
    AutoCompleteDirective.prototype.toggle = function ($event) {
        stopPropagationIfExist($event);
        if (this.popupRef) {
            var pop = this.popupRef.instance;
            if (pop.isOpen) {
                this.hidePopup();
                return;
            }
            this.onSourceChange(this.arraySource);
        }
    };
    AutoCompleteDirective.prototype.positionPopup = function () {
        var targetElement = this.popupRef.location.nativeElement;
        var hostElement = this.placementElement || this.elementRef.nativeElement;
        var clientRect = this.positionService.positionElements(hostElement, targetElement, this.placement, this.appendBody);
        this.renderer.setStyle(targetElement, 'left', clientRect.left + "px");
        this.renderer.setStyle(targetElement, 'top', clientRect.top + "px");
    };
    AutoCompleteDirective.prototype.fillPopup = function (source, term) {
        var _this = this;
        var pop = this.popupRef.instance;
        pop.source = source;
        pop.term = term;
        ['formatter', 'itemTemplate', 'noResultItemTemplate', 'cssClass']
            .forEach(function (key) {
            if (_this[key] !== undefined) {
                pop[key] = _this[key];
            }
        });
    };
    AutoCompleteDirective.prototype.writeInputValue = function (value) {
        this.renderer.setProperty(this.elementRef.nativeElement, 'value', this.formatter(value || '') || '');
    };
    AutoCompleteDirective.prototype.unSubscription = function () {
        if (this.subscription) {
            this.subscription.unsubscribe();
            this.subscription = null;
        }
    };
    AutoCompleteDirective.prototype.onTermChange = function (term) {
        this.value = term;
        if (this.popupRef) {
            this.popupRef.instance.term = term;
        }
        this.onChange(term);
    };
    AutoCompleteDirective.prototype.registerInputEvent = function (elementRef) {
        var _this = this;
        return fromEvent(elementRef.nativeElement, 'input')
            .map(function (e) { return e.target.value; })
            .do(function (term) { return _this.onTouched(); })
            .filter(function (term) { return !_this.disabled && _this.onSearch && term.length >= _this.minLength; })
            .debounceTime(this.delay)
            .do(function (term) { return _this.onTermChange(term); })
            .switchMap(function (term) { return _this.onSearch(term, _this); });
    };
    var AutoCompleteDirective_1;
    __decorate([
        Input(),
        __metadata("design:type", Boolean)
    ], AutoCompleteDirective.prototype, "disabled", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], AutoCompleteDirective.prototype, "cssClass", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Number)
    ], AutoCompleteDirective.prototype, "delay", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Number)
    ], AutoCompleteDirective.prototype, "minLength", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], AutoCompleteDirective.prototype, "appendBody", void 0);
    __decorate([
        Input(),
        __metadata("design:type", TemplateRef)
    ], AutoCompleteDirective.prototype, "itemTemplate", void 0);
    __decorate([
        Input(),
        __metadata("design:type", TemplateRef)
    ], AutoCompleteDirective.prototype, "noResultItemTemplate", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Function)
    ], AutoCompleteDirective.prototype, "formatter", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Function)
    ], AutoCompleteDirective.prototype, "valueParser", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Function)
    ], AutoCompleteDirective.prototype, "onSearch", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], AutoCompleteDirective.prototype, "selectValueChange", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], AutoCompleteDirective.prototype, "placementElement", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Array),
        __metadata("design:paramtypes", [Array])
    ], AutoCompleteDirective.prototype, "dataSource", null);
    __decorate([
        HostListener('blur', []),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], AutoCompleteDirective.prototype, "onBlur", null);
    __decorate([
        HostListener('keydown.esc', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], AutoCompleteDirective.prototype, "onEscKeyup", null);
    __decorate([
        HostListener('keydown.Enter', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], AutoCompleteDirective.prototype, "onEnterKeyDown", null);
    __decorate([
        HostListener('keydown.ArrowUp', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], AutoCompleteDirective.prototype, "onArrowUpKeyDown", null);
    __decorate([
        HostListener('keydown.ArrowDown', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Object]),
        __metadata("design:returntype", void 0)
    ], AutoCompleteDirective.prototype, "onArrowDownKeyDown", null);
    __decorate([
        HostListener('document:click', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Event]),
        __metadata("design:returntype", void 0)
    ], AutoCompleteDirective.prototype, "onDocumentClick", null);
    AutoCompleteDirective = AutoCompleteDirective_1 = __decorate([
        Directive({
            selector: '[reAutoComplete]',
            exportAs: 'autoComplete',
            host: {
                'autocomplete': 'off',
                'autocapitalize': 'off',
                'autocorrect': 'off'
            },
            providers: [{
                    provide: NG_VALUE_ACCESSOR,
                    useExisting: forwardRef(function () { return AutoCompleteDirective_1; }),
                    multi: true
                }]
        }),
        __metadata("design:paramtypes", [ElementRef, ViewContainerRef,
            ComponentFactoryResolver, Renderer2,
            Injector, PositionService,
            HiNGConfig, ChangeDetectorRef])
    ], AutoCompleteDirective);
    return AutoCompleteDirective;
}());
export { AutoCompleteDirective };
