import { ChangeDetectorRef, ElementRef, EventEmitter, OnDestroy, OnInit, TemplateRef } from '@angular/core';
import { ControlValueAccessor } from '@angular/forms';
import { ConnectedOverlayPositionChange } from '@angular/cdk/overlay';
import { UpdateHostClassService } from '../common/services/update-host-class.service';
export declare type CascaderExpandTrigger = 'click' | 'hover';
export declare type CascaderTriggerType = 'click' | 'hover';
export declare type CascaderSize = 'small' | 'large' | 'default';
export interface CascaderOption {
    value?: string;
    label?: string;
    title?: string;
    disabled?: boolean;
    loading?: boolean;
    isLeaf?: boolean;
    parent?: CascaderOption;
    children?: CascaderOption[];
    [key: string]: any;
}
export declare class CascaderComponent implements OnInit, OnDestroy, ControlValueAccessor {
    private elementRef;
    private cdr;
    private updateHostClassService;
    private allowClear;
    private autoFocus;
    private disabled;
    private enableCache;
    private showArrow;
    private showInput;
    private size;
    private prefixCls;
    private inputPrefixCls;
    private menuClassName;
    private columnClassName;
    private changeOnSelect;
    private defaultValue;
    dropDownPosition: string;
    menuVisible: boolean;
    isLoading: boolean;
    private isOpening;
    private _arrowCls;
    private _clearCls;
    private _inputCls;
    private _labelCls;
    private _loadingCls;
    private _menuCls;
    private _menuColumnCls;
    el: HTMLElement;
    private isFocused;
    private labelRenderTpl;
    isLabelRenderTemplate: boolean;
    labelRenderText: string;
    labelRenderContext: any;
    private value;
    private selectedOptions;
    private activatedOptions;
    hiColumns: CascaderOption[][];
    private delayTimer;
    private delaySelectTimer;
    private _inputValue;
    inputValue: string;
    onChange: any;
    onTouched: any;
    hiLabelRender: TemplateRef<any>;
    hiPrefixCls: string;
    hiDisabled: boolean;
    hiSize: CascaderSize;
    hiShowInput: boolean;
    hiAllowClear: boolean;
    hiAutoFocus: boolean;
    hiShowArrow: boolean;
    hiMenuClassName: string;
    hiColumnClassName: string;
    options: CascaderOption[] | null;
    hiChangeOnSelect: boolean;
    hiClearText: string;
    hiExpandTrigger: CascaderExpandTrigger;
    hiNotFoundContent: string;
    hiPlaceHolder: string;
    hiMenuStyle: {
        [key: string]: string;
    };
    hiChangeOn: (option: CascaderOption, level: number) => boolean;
    hiMouseEnterDelay: number;
    hiMouseLeaveDelay: number;
    hiTriggerAction: CascaderTriggerType | CascaderTriggerType[];
    hiValueProperty: string;
    hiLabelProperty: string;
    hiLoadData: (node: CascaderOption, index?: number) => PromiseLike<any>;
    hiVisibleChange: EventEmitter<boolean>;
    hiChange: EventEmitter<any[]>;
    hiSelectionChange: EventEmitter<CascaderOption[]>;
    hiSelect: EventEmitter<{
        option: CascaderOption;
        index: number;
    }>;
    hiClear: EventEmitter<any>;
    menu: ElementRef;
    onPositionChange(position: ConnectedOverlayPositionChange): void;
    focus(): void;
    blur(): void;
    private setClassMap;
    readonly labelCls: any;
    private setLabelClass;
    readonly arrowCls: any;
    private setArrowClass;
    readonly loadingCls: any;
    private setLoadingClass;
    readonly clearCls: any;
    private setClearClass;
    readonly inputCls: any;
    private setInputClass;
    readonly menuCls: any;
    private setMenuClass;
    readonly menuColumnCls: any;
    private setMenuColumnClass;
    getOptionCls(option: CascaderOption, index: number): any;
    handlerInputChange(event: Event): void;
    handleInputBlur(event: Event): void;
    handleInputFocus(event: Event): void;
    private hasInput;
    private hasValue;
    readonly showPlaceholder: boolean;
    readonly showClearIcon: boolean;
    clearSelection(event?: Event): void;
    private buildDisplayLabel;
    onKeyDown(event: KeyboardEvent): void;
    onTriggerClick(event: MouseEvent): void;
    onTriggerMouseEnter(event: MouseEvent): void;
    onTriggerMouseLeave(event: MouseEvent): void;
    private isClickTiggerAction;
    private isPointerTiggerAction;
    closeMenu(): void;
    private clearDelayTimer;
    delaySetMenuVisible(visible: boolean, delay: number, setOpening?: boolean): void;
    isMenuVisible(): boolean;
    setMenuVisible(menuVisible: boolean): void;
    private beforeVisible;
    private loadRootOptions;
    getOptionLabel(option: CascaderOption): any;
    getOptionValue(option: CascaderOption): any;
    private isActivedOption;
    private setActiveOption;
    private loadChildren;
    private onSelectOption;
    private isChangeOn;
    private setColumnData;
    onOptionClick(option: CascaderOption, index: number, event: Event): void;
    private onEnter;
    private moveUpOrDown;
    private moveUp;
    private moveDown;
    private moveLeft;
    private moveRight;
    onOptionMouseEnter(option: CascaderOption, index: number, event: Event): void;
    onOptionMouseLeave(option: CascaderOption, index: number, event: Event): void;
    private clearDelaySelectTimer;
    private delaySelect;
    getSubmitValue(): any[];
    private onValueChange;
    constructor(elementRef: ElementRef, cdr: ChangeDetectorRef, updateHostClassService: UpdateHostClassService);
    private findOption;
    private isLoaded;
    private activateOnInit;
    private initOptions;
    afterWriteValue(): void;
    writeValue(value: any): void;
    registerOnChange(fn: (_: any) => {}): void;
    registerOnTouched(fn: () => {}): void;
    setDisabledState(isDisabled: boolean): void;
    ngOnInit(): void;
    ngOnDestroy(): void;
}
