var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component, ChangeDetectionStrategy, Input, ChangeDetectorRef, forwardRef, Output, EventEmitter } from '@angular/core';
import { HiNGConfig } from '../hi.config';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
var CheckboxGroupComponent = (function () {
    function CheckboxGroupComponent(hiNGConfig, changeDetectorRef) {
        this.changeDetectorRef = changeDetectorRef;
        this.disabled = false;
        this.valueChange = new EventEmitter();
        this.value = [];
        this.onChange = function (_) { return null; };
        this.onTouched = function () { return null; };
        this.formatter = hiNGConfig.checkboxGroup.formatter;
        this.valueParser = hiNGConfig.checkboxGroup.valueParser;
    }
    CheckboxGroupComponent_1 = CheckboxGroupComponent;
    CheckboxGroupComponent.prototype.writeValue = function (obj) {
        this.value = obj instanceof Array ? obj : (obj ? [obj] : []);
        this.changeDetectorRef.markForCheck();
    };
    CheckboxGroupComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    CheckboxGroupComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    CheckboxGroupComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    CheckboxGroupComponent.prototype.selectAll = function () {
        var _this = this;
        this.value = this.options.map(function (item) { return _this.valueParser(item); });
        this.onChange(this.value);
        this.valueChange.emit(this.value);
    };
    CheckboxGroupComponent.prototype.unselectAll = function () {
        this.value = [];
        this.onChange(this.value);
        this.valueChange.emit(this.value);
    };
    CheckboxGroupComponent.prototype.invertSelect = function () {
        var _this = this;
        var current = this.value || [];
        this.value = this.options
            .filter(function (item) { return current.indexOf(_this.valueParser(item)) === -1; })
            .map(function (item) { return _this.valueParser(item); });
        this.onChange(this.value);
        this.valueChange.emit(this.value);
    };
    CheckboxGroupComponent.prototype.onCheckBoxChange = function (item, checkbox) {
        this.onTouched();
        this.value = this.value || [];
        var value = this.valueParser(item);
        if (checkbox.checked) {
            this.value = this.value.concat([value]);
        }
        else {
            this.value = this.value.filter(function (valueItem) { return valueItem !== value; });
        }
        this.onChange(this.value);
        this.valueChange.emit(this.value);
    };
    CheckboxGroupComponent.prototype.isChecked = function (item) {
        return this.value.indexOf(this.valueParser(item)) !== -1;
    };
    var CheckboxGroupComponent_1;
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], CheckboxGroupComponent.prototype, "disabled", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Array)
    ], CheckboxGroupComponent.prototype, "options", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Boolean)
    ], CheckboxGroupComponent.prototype, "inline", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], CheckboxGroupComponent.prototype, "cssClass", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Function)
    ], CheckboxGroupComponent.prototype, "formatter", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Function)
    ], CheckboxGroupComponent.prototype, "valueParser", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], CheckboxGroupComponent.prototype, "valueChange", void 0);
    CheckboxGroupComponent = CheckboxGroupComponent_1 = __decorate([
        Component({
            selector: 'hi-checkbox-group',
            template: "\n    <div [ngClass]=\"{'checkbox-inline': inline, 'checkbox': !inline}\" *ngFor=\"let item of options\"><label><input #checkbox type=\"checkbox\" [checked]=\"isChecked(item)\" [disabled]=\"disabled\" (change)=\"onCheckBoxChange(item, checkbox)\">{{formatter(item)}}</label></div>\n  ",
            changeDetection: ChangeDetectionStrategy.OnPush,
            exportAs: 'checkboxGroup',
            host: { '[class]': 'cssClass' },
            providers: [{
                    provide: NG_VALUE_ACCESSOR,
                    useExisting: forwardRef(function () { return CheckboxGroupComponent_1; }),
                    multi: true
                }]
        }),
        __metadata("design:paramtypes", [HiNGConfig, ChangeDetectorRef])
    ], CheckboxGroupComponent);
    return CheckboxGroupComponent;
}());
export { CheckboxGroupComponent };
