var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component, Input, HostListener, forwardRef, ElementRef, Renderer2, Output, EventEmitter, ChangeDetectorRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { SelectDateChangeReason } from './date-change-event-args.model';
import { HiNGConfig } from '../hi.config';
import { DefaultDateConverter } from '../utils/default-date-converter';
import { isValidDate } from '../utils/date-utils';
import { stopPropagationIfExist } from '../utils/dom-utils';
var DatePickerPopupComponent = (function () {
    function DatePickerPopupComponent(elementRef, renderer, hiNGConfig, changeDetectorRef) {
        this.elementRef = elementRef;
        this.renderer = renderer;
        this.hiNGConfig = hiNGConfig;
        this.changeDetectorRef = changeDetectorRef;
        this.selectedDateChange = new EventEmitter();
        this.disabled = false;
        this.onChange = function (_) { return null; };
        this.onTouched = function () { return null; };
        this.locale = this.hiNGConfig.datePicker.locale;
        this.dateConverter = hiNGConfig.datePicker.dateConverter || new DefaultDateConverter();
        this.dateConfig = hiNGConfig.datePicker;
        this.showTimePicker = hiNGConfig.datePicker.timePicker;
        this._minDate = new Date(this.dateConfig.min, 0, 1, 0, 0, 0);
        this._maxDate = new Date(this.dateConfig.max, 11, 31, 23, 59, 59);
        this.renderer.setStyle(this.elementRef.nativeElement, 'display', 'inline-block');
    }
    DatePickerPopupComponent_1 = DatePickerPopupComponent;
    Object.defineProperty(DatePickerPopupComponent.prototype, "maxDate", {
        get: function () {
            return this._maxDate;
        },
        set: function (date) {
            var parseDate = this.dateConverter.parse(date, this.dateFormat, this.locale);
            if (parseDate) {
                this._maxDate = parseDate;
                this.onYearRangeChange();
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DatePickerPopupComponent.prototype, "minDate", {
        get: function () {
            return this._minDate;
        },
        set: function (date) {
            var parseDate = this.dateConverter.parse(date, this.dateFormat, this.locale);
            if (parseDate) {
                this._minDate = parseDate;
                this.onYearRangeChange();
            }
        },
        enumerable: true,
        configurable: true
    });
    DatePickerPopupComponent.prototype.ngOnInit = function () {
        this.hourOptions = this.fillRange(24);
        this.minuteOptions = this.fillRange(60);
        this.secondOptions = this.fillRange(60);
        this.onSelectDateChanged();
        this.onDisplayWeeksChange();
        this.onYearRangeChange();
    };
    DatePickerPopupComponent.prototype.fillRange = function (number) {
        var _this = this;
        return new Array(number).fill(0).map(function (value, index) { return _this.fillLeft(index); });
    };
    DatePickerPopupComponent.prototype.writeValue = function (obj) {
        this.selectedDate = obj ? this.dateConverter.parse(obj, this.dateFormat, this.locale) : null;
        this.onSelectDateChanged();
        this.onDisplayWeeksChange();
        this.changeDetectorRef.markForCheck();
    };
    DatePickerPopupComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    DatePickerPopupComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    DatePickerPopupComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    DatePickerPopupComponent.prototype.todayIsOutOfRange = function () {
        return this.isDisabledDay(new Date());
    };
    DatePickerPopupComponent.prototype.today = function ($event) {
        this.onSelectDate($event, new Date());
    };
    DatePickerPopupComponent.prototype.onSelectDate = function ($event, date) {
        stopPropagationIfExist($event);
        if (this.isDisabledDay(date)) {
            return;
        }
        var selectedDate = new Date(date.getFullYear(), date.getMonth(), date.getDate(), this.currentHour, this.currentMinute, this.currentSecond);
        this.onTouched();
        this.writeValue(selectedDate);
        this.onChange(selectedDate.getTime());
        this.selectedDateChange.emit({ reason: SelectDateChangeReason.date, date: this.selectedDate });
        if (this.currentMonth !== this.selectedDate.getMonth() || this.currentYear !== this.selectedDate.getFullYear()) {
            this.currentYear = this.selectedDate.getFullYear();
            this.currentMonth = this.selectedDate.getMonth();
            this.onDisplayWeeksChange();
        }
    };
    DatePickerPopupComponent.prototype.onTimeChange = function () {
        var date = this.selectedDate || new Date();
        this.selectedDate = new Date(date.getFullYear(), date.getMonth(), date.getDate(), this.currentHour, this.currentMinute, this.currentSecond);
        this.onTouched();
        this.writeValue(this.selectedDate);
        this.onChange(this.selectedDate.getTime());
        this.selectedDateChange.emit({ reason: SelectDateChangeReason.time, date: this.selectedDate });
    };
    DatePickerPopupComponent.prototype.hasPreMonth = function () {
        return this.currentMonth > 0 || this.currentYear > this.minDate.getFullYear();
    };
    DatePickerPopupComponent.prototype.onPreMonth = function () {
        if (!this.hasPreMonth()) {
            return;
        }
        if (this.currentMonth > 0) {
            this.currentMonth -= 1;
        }
        else {
            this.currentMonth = 11;
            this.currentYear -= 1;
        }
        this.onDisplayWeeksChange();
    };
    DatePickerPopupComponent.prototype.hasNextMonth = function () {
        return this.currentMonth < 11 || this.currentYear < this.maxDate.getFullYear();
    };
    DatePickerPopupComponent.prototype.onNextMonth = function () {
        if (!this.hasNextMonth()) {
            return;
        }
        if (this.currentMonth < 11) {
            this.currentMonth += 1;
        }
        else {
            this.currentMonth = 0;
            this.currentYear += 1;
        }
        this.onDisplayWeeksChange();
    };
    DatePickerPopupComponent.prototype.isDisabledDay = function (date) {
        var minDate = new Date(this.minDate.getFullYear(), this.minDate.getMonth(), this.minDate.getDate(), 0, 0, 0);
        var maxDate = new Date(this.maxDate.getFullYear(), this.maxDate.getMonth(), this.maxDate.getDate(), 23, 59, 59);
        return this.disabled || (date.getTime() < minDate.getTime() ||
            date.getTime() > maxDate.getTime());
    };
    DatePickerPopupComponent.prototype.isSelectDay = function (date) {
        if (!this.selectedDate || !date) {
            return false;
        }
        return date.getFullYear() === this.selectedDate.getFullYear() &&
            date.getMonth() === this.selectedDate.getMonth() &&
            date.getDate() === this.selectedDate.getDate();
    };
    DatePickerPopupComponent.prototype.onYearRangeChange = function () {
        if (!isValidDate(this.minDate) || !isValidDate(this.maxDate)) {
            return;
        }
        var minYear = this.minDate.getFullYear();
        var maxYear = this.maxDate.getFullYear();
        this.yearOptions = new Array(maxYear - minYear + 1).fill(0).map(function (value, index) {
            return minYear + index;
        });
    };
    DatePickerPopupComponent.prototype.onHostClick = function ($event) {
        stopPropagationIfExist($event);
    };
    DatePickerPopupComponent.prototype.onSelectDateChanged = function () {
        var date = this.selectedDate || new Date();
        if (date.getTime() < this.minDate.getTime()) {
            date = this.minDate;
        }
        if (date.getTime() > this.maxDate.getTime()) {
            date = this.maxDate;
        }
        this.currentYear = date.getFullYear();
        this.currentMonth = date.getMonth();
        this.currentHour = this.showTimePicker ? date.getHours() : 0;
        this.currentMinute = this.showTimePicker ? date.getMinutes() : 0;
        this.currentSecond = this.showTimePicker ? date.getSeconds() : 0;
    };
    DatePickerPopupComponent.prototype.fillLeft = function (num) {
        return num < 10 ? "0" + num : "" + num;
    };
    DatePickerPopupComponent.prototype.onDisplayWeeksChange = function () {
        var _this = this;
        var firstDayOfMonth = new Date(this.currentYear, this.currentMonth, 1);
        var weekOfDay = firstDayOfMonth.getDay();
        var startDate = new Date(firstDayOfMonth.getTime() - weekOfDay * DatePickerPopupComponent_1.DAY_DURATION);
        var displayWeeks = [];
        var _loop_1 = function (i) {
            var startWeekDate = startDate.getTime() + i * 7 * DatePickerPopupComponent_1.DAY_DURATION;
            var weekDays = new Array(7).fill(0).map(function (value, index) {
                var currentDate = new Date(startWeekDate + index * DatePickerPopupComponent_1.DAY_DURATION);
                return {
                    day: _this.fillLeft(currentDate.getDate()),
                    date: currentDate,
                    inMonth: currentDate.getMonth().toString() === _this.currentMonth.toString()
                };
            });
            displayWeeks.push(weekDays);
        };
        for (var i = 0; i < 6; i++) {
            _loop_1(i);
        }
        this.displayWeeks = displayWeeks;
    };
    var DatePickerPopupComponent_1;
    DatePickerPopupComponent.DAY_DURATION = 24 * 60 * 60 * 1000;
    __decorate([
        Input(),
        __metadata("design:type", Boolean)
    ], DatePickerPopupComponent.prototype, "showTimePicker", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], DatePickerPopupComponent.prototype, "cssClass", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], DatePickerPopupComponent.prototype, "dateConverter", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], DatePickerPopupComponent.prototype, "locale", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], DatePickerPopupComponent.prototype, "selectedDateChange", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], DatePickerPopupComponent.prototype, "dateFormat", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], DatePickerPopupComponent.prototype, "disabled", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], DatePickerPopupComponent.prototype, "maxDate", null);
    __decorate([
        Input(),
        __metadata("design:type", Object),
        __metadata("design:paramtypes", [Object])
    ], DatePickerPopupComponent.prototype, "minDate", null);
    __decorate([
        HostListener('click', ['$event']),
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Event]),
        __metadata("design:returntype", void 0)
    ], DatePickerPopupComponent.prototype, "onHostClick", null);
    DatePickerPopupComponent = DatePickerPopupComponent_1 = __decorate([
        Component({
            selector: 'hi-date-picker-popup',
            template: "\n    <div class=\"month-view {{cssClass}}\"><table class=\"table month-view-table\"><thead><tr><td><a *ngIf=\"hasPreMonth()\" class=\"btn-link icmn-backward2\" aria-hidden=\"true\" (click)=\"onPreMonth()\"></a></td><td colspan=\"5\" class=\"text-center\"><select class=\"date-select\" [(ngModel)]=\"currentMonth\" (change)=\"onDisplayWeeksChange()\"><option *ngFor=\"let item of dateConfig.months; let $index = index\" [ngValue]=\"$index\">{{item}}</option></select><select class=\"date-select\" [(ngModel)]=\"currentYear\" (change)=\"onDisplayWeeksChange()\"><option *ngFor=\"let item of yearOptions\" [ngValue]=\"item\">{{item}}</option></select></td><td><a *ngIf=\"hasNextMonth()\" class=\"btn-link icmn-forward3\" aria-hidden=\"true\" (click)=\"onNextMonth()\"></a></td></tr><tr class=\"small text-center week-header\"><td *ngFor=\"let item of dateConfig.weeks\">{{item}}</td></tr></thead><tbody><tr *ngFor=\"let week of displayWeeks\"><td *ngFor=\"let day of week\" class=\"day\" [ngClass]=\"{'out-of-month': !day.inMonth, 'in-month-day': day.inMonth,'disabled': isDisabledDay(day.date)}\" (click)=\"onSelectDate($event, day.date)\"><div class=\"calendar-date\" [ngClass]=\"{'bg-primary': isSelectDay(day.date)}\">{{day.day}}</div></td></tr></tbody><tfoot><tr class=\"time-picker-view bg-info\"><td colspan=\"7\"><div *ngIf=\"showTimePicker\" class=\"pull-left time-panel\"><select class=\"date-select\" [(ngModel)]=\"currentHour\" name=\"hours\" [disabled]=\"disabled\" (change)=\"onTimeChange()\"><option *ngFor=\"let item of hourOptions; let $index = index\" [ngValue]=\"$index\">{{item}}</option></select>:<select class=\"date-select\" [(ngModel)]=\"currentMinute\" name=\"minutes\" [disabled]=\"disabled\" (change)=\"onTimeChange()\"><option *ngFor=\"let item of minuteOptions; let $index = index\" [ngValue]=\"$index\">{{item}}</option></select>:<select class=\"date-select\" [(ngModel)]=\"currentSecond\" name=\"seconds\" [disabled]=\"disabled\" (change)=\"onTimeChange()\"><option *ngFor=\"let item of secondOptions; let $index = index\" [ngValue]=\"$index\">{{item}}</option></select></div><button type=\"button\" class=\"btn btn-sm btn-primary pull-right\" [disabled]=\"todayIsOutOfRange()\" (click)=\"today($event)\">{{dateConfig.today}}</button></td></tr></tfoot></table></div>\n  ",
            exportAs: 'datePickerPopup',
            providers: [{
                    provide: NG_VALUE_ACCESSOR,
                    useExisting: forwardRef(function () { return DatePickerPopupComponent_1; }),
                    multi: true
                }]
        }),
        __metadata("design:paramtypes", [ElementRef,
            Renderer2,
            HiNGConfig,
            ChangeDetectorRef])
    ], DatePickerPopupComponent);
    return DatePickerPopupComponent;
}());
export { DatePickerPopupComponent };
