var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { ViewChild, ElementRef, Input, Output, EventEmitter, TemplateRef, } from '@angular/core';
import { readFileAsDataURL } from '../utils/dom-utils';
import { formatFileSize, formatString } from '../utils/lange-utils';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/catch';
import { of } from 'rxjs/observable/of';
import { forkJoin } from 'rxjs/observable/forkJoin';
import { noop } from '../utils';
var FileUpload = (function () {
    function FileUpload(hiNGConfig, renderer, http, changeDetectorRef) {
        this.hiNGConfig = hiNGConfig;
        this.renderer = renderer;
        this.http = http;
        this.changeDetectorRef = changeDetectorRef;
        this.multiple = true;
        this.showErrors = true;
        this.selectFilesChange = new EventEmitter();
        this.fileUploadStart = new EventEmitter();
        this.fileUploadCompleted = new EventEmitter();
        this.fileUploadSuccess = new EventEmitter();
        this.fileUploadError = new EventEmitter();
        this.removeFiles = new EventEmitter();
        this.uploadFilesChange = new EventEmitter();
        this.uploadFiles = [];
        this.selectFiles = [];
        this.errors = [];
        this.fileSizeErrorMessage = this.hiNGConfig.fileUpload.fileSizeErrorMessage;
        this.fileTypeErrorMessage = this.hiNGConfig.fileUpload.fileTypeErrorMessage;
        this.uploadParamName = this.hiNGConfig.fileUpload.uploadParamName;
        this.previewWidth = this.hiNGConfig.fileUpload.previewWidth;
        this.previewHeight = this.hiNGConfig.fileUpload.previewHeight;
        this.imgPreview = this.hiNGConfig.fileUpload.imgPreview;
        this.chooseButton = this.hiNGConfig.fileUpload.chooseButton;
        this.uploadButton = this.hiNGConfig.fileUpload.uploadButton;
        this.cancelButton = this.hiNGConfig.fileUpload.cancelButton;
        this.plusIcon = this.hiNGConfig.fileUpload.plusIcon;
        this.uploadIcon = this.hiNGConfig.fileUpload.uploadIcon;
        this.loadingIcon = this.hiNGConfig.fileUpload.loadingIcon;
        this.removeIcon = this.hiNGConfig.fileUpload.removeIcon;
        this.showErrors = this.hiNGConfig.fileUpload.showErrors;
    }
    FileUpload.prototype.ngAfterViewInit = function () {
        if (this.accept) {
            this.renderer.setProperty(this.fileInput.nativeElement, 'accept', this.accept);
        }
        this.renderer.setProperty(this.fileInput.nativeElement, 'multiple', this.multiple);
    };
    FileUpload.prototype.isMoreThanMaxItems = function () {
        var fileSize = this.getFileCount();
        if (!this.multiple) {
            return fileSize >= 1;
        }
        return this.maxItems && (fileSize >= this.maxItems);
    };
    FileUpload.prototype.clearErrors = function () {
        this.errors = [];
    };
    FileUpload.prototype.onDropFiles = function ($event) {
        $event.stopPropagation();
        $event.preventDefault();
        this.clearErrors();
        var files = $event.dataTransfer.files;
        if (files && files.length) {
            this.handleFileChoose(files);
        }
    };
    FileUpload.prototype.addNewFile = function ($event) {
        $event.stopPropagation();
        if (!this.isMoreThanMaxItems()) {
            this.fileInput.nativeElement.value = null;
            this.fileInput.nativeElement.click();
        }
    };
    FileUpload.prototype.newFileChoose = function (fileInput) {
        this.clearErrors();
        if (fileInput.files && fileInput.files.length) {
            this.handleFileChoose(fileInput.files);
        }
    };
    FileUpload.prototype.removeAllSelectedFiles = function () {
        var files = this.selectFiles;
        this.selectFiles = [];
        this.clearErrors();
        this.removeFiles.emit(files);
    };
    FileUpload.prototype.onRemoveFile = function (fileItem) {
        this.selectFiles = this.selectFiles.filter(function (item) { return item !== fileItem; });
        this.removeFiles.emit([fileItem]);
    };
    FileUpload.prototype.onRemoveUploadFile = function (fileItem) {
        this.uploadFiles = this.uploadFiles.filter(function (item) { return item !== fileItem; });
        this.removeFiles.emit([fileItem]);
        this.uploadFilesChange.emit(this.uploadFiles);
    };
    FileUpload.prototype.uploadAllFiles = function () {
        this.clearErrors();
        this.httpUploadAllFile(this.selectFiles);
    };
    FileUpload.prototype.httpUploadAllFile = function (files) {
        var _this = this;
        this.isUploading = true;
        this.fileUploadStart.emit(files);
        var subscriptions = files.map(function (fileItem) { return _this.httpUploadFile(fileItem); });
        forkJoin(subscriptions)
            .subscribe(noop, noop, function () {
            _this.isUploading = false;
            _this.fileUploadCompleted.emit(files);
        });
    };
    FileUpload.prototype.httpUploadFile = function (fileItem) {
        var _this = this;
        var formData = new FormData();
        formData.append(this.uploadParamName, fileItem.file);
        return this.http.post(this.uploadUrl, formData, this.uploadRequestOptions)
            .map(function (res) { return _this.onFileUploadSuccess(fileItem, res); })
            .catch(function (error) { return _this.onFileUploadError(fileItem, error); });
    };
    FileUpload.prototype.onFileUploadSuccess = function (fileItem, res) {
        fileItem.uploadResponse = res;
        this.selectFiles = this.selectFiles.filter(function (item) { return item !== fileItem; });
        this.uploadFiles = (this.uploadFiles || []).concat([fileItem]);
        this.fileUploadSuccess.emit(fileItem);
        this.uploadFilesChange.emit(this.uploadFiles);
        this.changeDetectorRef.markForCheck();
        return of({ result: res, success: true });
    };
    FileUpload.prototype.onFileUploadError = function (fileItem, error) {
        this.errors.push(fileItem.name + ": " + (error.error || error.statusText));
        this.fileUploadError.emit({
            name: fileItem.name,
            displaySize: fileItem.displaySize,
            dataUrl: fileItem.dataUrl,
            file: fileItem.file,
            uploadResponse: error
        });
        this.changeDetectorRef.markForCheck();
        return of({ error: error, success: false });
    };
    FileUpload.prototype.handleFileChoose = function (uploadFiles) {
        var _this = this;
        var files = this.validFiles(Array.from(uploadFiles));
        this.mapFileModel(files)
            .then(function (fileModels) {
            _this.selectFiles = _this.selectFiles.concat(fileModels);
            _this.selectFilesChange.emit(_this.selectFiles);
            _this.changeDetectorRef.markForCheck();
            return fileModels || [];
        })
            .then(function (fileModels) {
            if (_this.autoUpload) {
                return _this.httpUploadAllFile(fileModels);
            }
        });
    };
    FileUpload.prototype.validFiles = function (files) {
        var _this = this;
        var fileCount = this.getFileCount();
        var size = this.multiple ? this.maxItems : 1;
        if (size && (fileCount + files.length > size)) {
            files = files.slice(0, size - fileCount);
        }
        return files.filter(function (file) {
            return _this.validFile(file);
        });
    };
    FileUpload.prototype.validFile = function (file) {
        var _a;
        var errors = [];
        if (this.maxFileSize && file.size > this.maxFileSize) {
            errors.push(formatString(this.fileSizeErrorMessage, file.name, formatFileSize(this.maxFileSize)));
        }
        if (this.accept && !this.validFileType(file)) {
            errors.push(formatString(this.fileTypeErrorMessage, file.name, this.accept));
        }
        (_a = this.errors).push.apply(_a, errors);
        this.fileUploadError.emit({
            name: file.name,
            file: file,
            uploadResponse: this.errors
        });
        return !errors.length;
    };
    FileUpload.prototype.getFileCount = function () {
        return ((this.selectFiles || []).length + (this.uploadFiles || []).length);
    };
    FileUpload.prototype.validFileType = function (file) {
        return this.accept.split(',').some(function (type) {
            return new RegExp("^" + type.replace(/\*/g, '.*') + "$").test(file.type);
        });
    };
    FileUpload.prototype.mapFileModel = function (files) {
        return Promise.all(files.map(function (file) {
            return readFileAsDataURL(file)
                .then(function (dataUrl) { return ({ dataUrl: dataUrl, name: file.name, file: file, displaySize: formatFileSize(file.size) }); });
        }));
    };
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], FileUpload.prototype, "accept", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], FileUpload.prototype, "multiple", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Boolean)
    ], FileUpload.prototype, "autoUpload", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], FileUpload.prototype, "showErrors", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Number)
    ], FileUpload.prototype, "maxItems", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Number)
    ], FileUpload.prototype, "maxFileSize", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], FileUpload.prototype, "uploadUrl", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], FileUpload.prototype, "uploadParamName", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], FileUpload.prototype, "uploadRequestOptions", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Boolean)
    ], FileUpload.prototype, "imgPreview", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], FileUpload.prototype, "previewWidth", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], FileUpload.prototype, "previewHeight", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], FileUpload.prototype, "cssClass", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], FileUpload.prototype, "fileSizeErrorMessage", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], FileUpload.prototype, "fileTypeErrorMessage", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], FileUpload.prototype, "chooseButton", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], FileUpload.prototype, "uploadButton", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], FileUpload.prototype, "cancelButton", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], FileUpload.prototype, "plusIcon", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], FileUpload.prototype, "uploadIcon", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], FileUpload.prototype, "loadingIcon", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], FileUpload.prototype, "removeIcon", void 0);
    __decorate([
        Input(),
        __metadata("design:type", TemplateRef)
    ], FileUpload.prototype, "toolbarTemplate", void 0);
    __decorate([
        Input(),
        __metadata("design:type", TemplateRef)
    ], FileUpload.prototype, "previewTemplate", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], FileUpload.prototype, "selectFilesChange", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], FileUpload.prototype, "fileUploadStart", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], FileUpload.prototype, "fileUploadCompleted", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], FileUpload.prototype, "fileUploadSuccess", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], FileUpload.prototype, "fileUploadError", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], FileUpload.prototype, "removeFiles", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], FileUpload.prototype, "uploadFilesChange", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Array)
    ], FileUpload.prototype, "uploadFiles", void 0);
    __decorate([
        ViewChild('file', { static: false }),
        __metadata("design:type", ElementRef)
    ], FileUpload.prototype, "fileInput", void 0);
    return FileUpload;
}());
export { FileUpload };
