import { Observable } from 'rxjs/Observable';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/catch';
import 'rxjs/add/observable/throw';
import { HttpClient, HttpRequest, HttpResponse, HttpParams, HttpHandler, HttpHeaders, HttpInterceptor, HttpEvent, HttpSentEvent, HttpUserEvent, HttpProgressEvent, HttpErrorResponse, HttpHeaderResponse } from '@angular/common/http';
export interface HttpInterceptord {
    request?: (option: HttpRequest<any>) => HttpRequest<any> | void;
    response?: (response: HttpEvent<any> | HttpErrorResponse, request?: HttpRequest<any>) => HttpEvent<any> | void;
}
export declare class HttpProvider {
    private interceptors;
    constructor();
    getInterceptors(): HttpInterceptord[];
    addInterceptor(interceptor: HttpInterceptord): HttpProvider;
    addRequestInterceptor(interceptor: (res: HttpRequest<any>) => HttpRequest<any>): HttpProvider;
    addResponseInterceptor(interceptor: (res: any, request?: HttpRequest<any>) => any): HttpProvider;
    addResponseErrorInterceptor(interceptor: (res: any, request?: HttpRequest<any>) => any): HttpProvider;
    handleRequest(req: HttpRequest<any>): HttpRequest<any>;
    handleResponse(response: HttpEvent<any>, request?: HttpRequest<any>): HttpEvent<any>;
    baseUrl(host: string, excludes?: RegExp[]): HttpProvider;
    headers(headers?: {
        [name: string]: string | string[];
    }): HttpProvider;
}
export declare class HttpInterceptors implements HttpInterceptor {
    private httpProvider;
    constructor(httpProvider: HttpProvider);
    intercept(req: HttpRequest<any>, next: HttpHandler): Observable<HttpSentEvent | HttpHeaderResponse | HttpProgressEvent | HttpResponse<any> | HttpUserEvent<any>>;
}
export declare class Httpd {
    protected http: HttpClient;
    constructor(http: HttpClient);
    protected getBaseUrl(): string;
    protected getHeaders(): Object;
    protected setBaseUrl(url: string): () => string;
    protected setHeaders(headers?: any): () => any;
}
export declare function BaseUrl(url: string): <TFunction extends Function>(target: TFunction) => TFunction;
export declare function Headers(headers: any): <TFunction extends Function>(target: TFunction) => TFunction;
export declare const Path: (key?: string) => (target: Httpd, propertyKey: string, parameterIndex: number) => void;
export declare const Query: (key?: string) => (target: Httpd, propertyKey: string, parameterIndex: number) => void;
export declare const Body: (target: Httpd, propertyKey: string, parameterIndex: number) => void;
export declare const Header: (key?: string) => (target: Httpd, propertyKey: string, parameterIndex: number) => void;
export declare function RequestOptions(options: {
    headers?: HttpHeaders | {
        [header: string]: string | string[];
    };
    observe?: 'body' | 'response' | string;
    params?: HttpParams | {
        [param: string]: string | string[];
    };
    reportProgress?: boolean;
    responseType?: 'arraybuffer' | 'blob' | 'json' | 'text';
    withCredentials?: boolean;
}): (target: Httpd, propertyKey: string, descriptor: any) => any;
export declare const GET: (url: string) => (target: Httpd, propertyKey: string, descriptor: any) => any;
export declare const JSONP: (url: string) => (target: Httpd, propertyKey: string, descriptor: any) => any;
export declare const POST: (url: string) => (target: Httpd, propertyKey: string, descriptor: any) => any;
export declare const PUT: (url: string) => (target: Httpd, propertyKey: string, descriptor: any) => any;
export declare const DELETE: (url: string) => (target: Httpd, propertyKey: string, descriptor: any) => any;
export declare const HEAD: (url: string) => (target: Httpd, propertyKey: string, descriptor: any) => any;
export declare const PATCH: (url: string) => (target: Httpd, propertyKey: string, descriptor: any) => any;
export declare const OPTIONS: (url: string) => (target: Httpd, propertyKey: string, descriptor: any) => any;
export declare const HTTP_PROVIDERS: any[];
