var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { animate, state, style, transition, trigger } from '@angular/animations';
import { Component, Input, ViewEncapsulation } from '@angular/core';
import { MessageContainerComponent } from './message-container.component';
var MessageComponent = (function () {
    function MessageComponent(_messageContainer) {
        this._messageContainer = _messageContainer;
        this._eraseTimer = null;
    }
    MessageComponent.prototype.ngOnInit = function () {
        this._options = this.hiMessage.options;
        if (this._options.hiAnimate) {
            this.hiMessage.state = 'enter';
        }
        this._autoErase = this._options.hiDuration > 0;
        if (this._autoErase) {
            this._initErase();
            this._startEraseTimeout();
        }
    };
    MessageComponent.prototype.ngOnDestroy = function () {
        if (this._autoErase) {
            this._clearEraseTimeout();
        }
    };
    MessageComponent.prototype.onEnter = function () {
        if (this._autoErase && this._options.hiPauseOnHover) {
            this._clearEraseTimeout();
            this._updateTTL();
        }
    };
    MessageComponent.prototype.onLeave = function () {
        if (this._autoErase && this._options.hiPauseOnHover) {
            this._startEraseTimeout();
        }
    };
    MessageComponent.prototype._destroy = function () {
        var _this = this;
        if (this._options.hiAnimate) {
            this.hiMessage.state = 'leave';
            setTimeout(function () { return _this._messageContainer.removeMessage(_this.hiMessage.messageId); }, 200);
        }
        else {
            this._messageContainer.removeMessage(this.hiMessage.messageId);
        }
    };
    MessageComponent.prototype._initErase = function () {
        this._eraseTTL = this._options.hiDuration;
        this._eraseTimingStart = Date.now();
    };
    MessageComponent.prototype._updateTTL = function () {
        if (this._autoErase) {
            this._eraseTTL -= Date.now() - this._eraseTimingStart;
        }
    };
    MessageComponent.prototype._startEraseTimeout = function () {
        var _this = this;
        if (this._eraseTTL > 0) {
            this._clearEraseTimeout();
            this._eraseTimer = window.setTimeout(function () { return _this._destroy(); }, this._eraseTTL);
            this._eraseTimingStart = Date.now();
        }
        else {
            this._destroy();
        }
    };
    MessageComponent.prototype._clearEraseTimeout = function () {
        if (this._eraseTimer !== null) {
            window.clearTimeout(this._eraseTimer);
            this._eraseTimer = null;
        }
    };
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], MessageComponent.prototype, "hiMessage", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Number)
    ], MessageComponent.prototype, "hiIndex", void 0);
    MessageComponent = __decorate([
        Component({
            selector: 'hi-message',
            preserveWhitespaces: false,
            animations: [
                trigger('enterLeave', [
                    state('enter', style({ opacity: 1, transform: 'translateY(0)' })),
                    transition('* => enter', [
                        style({ opacity: 0, transform: 'translateY(-50%)' }),
                        animate('100ms linear')
                    ]),
                    state('leave', style({ opacity: 0, transform: 'translateY(-50%)' })),
                    transition('* => leave', [
                        style({ opacity: 1, transform: 'translateY(0)' }),
                        animate('100ms linear')
                    ])
                ])
            ],
            template: "\n    <div class=\"hi-message-notice\" [@enterLeave]=\"hiMessage.state\" (mouseenter)=\"onEnter()\" (mouseleave)=\"onLeave()\"><div class=\"hi-message-notice-content\"><div class=\"hi-message-custom-content\" [ngClass]=\"'hi-message-' + hiMessage.type\"><ng-container [ngSwitch]=\"hiMessage.type\"><i *ngSwitchCase=\"'success'\" class=\"anticon anticon-check-circle\"></i> <i *ngSwitchCase=\"'info'\" class=\"anticon anticon-info-circle\"></i> <i *ngSwitchCase=\"'warning'\" class=\"anticon anticon-exclamation-circle\"></i> <i *ngSwitchCase=\"'error'\" class=\"anticon anticon-cross-circle\"></i> <i *ngSwitchCase=\"'loading'\" class=\"anticon anticon-spin anticon-loading\"></i></ng-container><span [innerHTML]=\"hiMessage.content\"></span></div></div></div>\n  ",
            styles: ["\n    @font-face{font-family:anticon;src:url(https://at.alicdn.com/t/font_zck90zmlh7hf47vi.eot);src:url(https://at.alicdn.com/t/font_zck90zmlh7hf47vi.eot#iefix) format(\"embedded-opentype\"),url(https://at.alicdn.com/t/font_zck90zmlh7hf47vi.woff) format(\"woff\"),url(https://at.alicdn.com/t/font_zck90zmlh7hf47vi.ttf) format(\"truetype\"),url(https://at.alicdn.com/t/font_zck90zmlh7hf47vi.svg#iconfont) format(\"svg\")}.anticon{display:inline-block;font-style:normal;vertical-align:baseline;text-align:center;text-transform:none;line-height:1;text-rendering:optimizeLegibility;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.anticon::before{display:block;font-family:anticon!important}.anticon-info-circle:before{content:\"\\e628\"}.anticon-info-circle-o:before{content:\"\\e629\"}.anticon-check-circle:before{content:\"\\e630\"}.anticon-check-circle-o:before{content:\"\\e631\"}.anticon-exclamation-circle:before{content:\"\\e62c\"}.anticon-exclamation-circle-o:before{content:\"\\e62d\"}.anticon-cross-circle:before{content:\"\\e62e\"}.anticon-cross-circle-o:before{content:\"\\e62f\"}.anticon-loading:before{content:\"\\e64d\"}.anticon-spin:before{display:inline-block;-webkit-animation:1s linear infinite loadingCircle;animation:1s linear infinite loadingCircle}.hi-message{font-family:\"Monospaced Number\",\"Chinese Quote\",-apple-system,BlinkMacSystemFont,\"Segoe UI\",Roboto,\"PingFang SC\",\"Hiragino Sans GB\",\"Microsoft YaHei\",\"Helvetica Neue\",Helvetica,Arial,sans-serif;font-size:14px;line-height:1.5;color:rgba(0,0,0,.65);-webkit-box-sizing:border-box;box-sizing:border-box;margin:0;padding:0;list-style:none;position:fixed;z-index:1010;width:100%;top:16px;left:0;pointer-events:none}.hi-message-notice{padding:8px;text-align:center}.hi-message-notice:first-child{margin-top:-8px}.hi-message .anticon{margin-right:8px;font-size:16px;top:1px;position:relative}.hi-message-notice-content{padding:10px 16px;border-radius:4px;-webkit-box-shadow:0 4px 12px rgba(0,0,0,.15);box-shadow:0 4px 12px rgba(0,0,0,.15);background:#fff;display:inline-block;pointer-events:all}.hi-message-success .anticon{color:#52c41a}.hi-message-error .anticon{color:#f5222d}.hi-message-warning .anticon{color:#faad14}.hi-message-info .anticon,.hi-message-loading .anticon{color:#28a745}.hi-message-notice.move-up-leave.move-up-leave-active{-webkit-animation-name:MessageMoveOut;animation-name:MessageMoveOut;overflow:hidden;-webkit-animation-duration:.3s;animation-duration:.3s}@-webkit-keyframes MessageMoveOut{0%{opacity:1;max-height:150px;padding:8px}100%{opacity:0;max-height:0;padding:0}}@keyframes MessageMoveOut{0%{opacity:1;max-height:150px;padding:8px}100%{opacity:0;max-height:0;padding:0}}@keyframes loadingCircle{0%{-webkit-transform-origin:50% 50%;transform-origin:50% 50%;-webkit-transform:rotate(0);transform:rotate(0)}100%{-webkit-transform-origin:50% 50%;transform-origin:50% 50%;-webkit-transform:rotate(360deg);transform:rotate(360deg)}}@-webkit-keyframes loadingCircle{0%{-webkit-transform-origin:50% 50%;transform-origin:50% 50%;-webkit-transform:rotate(0);transform:rotate(0)}100%{-webkit-transform-origin:50% 50%;transform-origin:50% 50%;-webkit-transform:rotate(360deg);transform:rotate(360deg)}}\n  "],
            encapsulation: ViewEncapsulation.None,
        }),
        __metadata("design:paramtypes", [MessageContainerComponent])
    ], MessageComponent);
    return MessageComponent;
}());
export { MessageComponent };
