var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component, Input, EventEmitter, ChangeDetectionStrategy, forwardRef, ChangeDetectorRef } from '@angular/core';
import { HiNGConfig } from '../hi.config';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
var RatingComponent = (function () {
    function RatingComponent(hiNGConfig, changeDetectorRef) {
        this.hiNGConfig = hiNGConfig;
        this.changeDetectorRef = changeDetectorRef;
        this.value = 0;
        this.disabled = false;
        this.valueChange = new EventEmitter();
        this.statValue = 0;
        this.onChange = function (_) { return null; };
        this.onTouched = function () { return null; };
        this.icons = hiNGConfig.rating.icons;
        this.ratingRanges = this.fillRatingRange(hiNGConfig.rating.max);
    }
    RatingComponent_1 = RatingComponent;
    Object.defineProperty(RatingComponent.prototype, "max", {
        set: function (value) {
            this.ratingRanges = this.fillRatingRange(value);
        },
        enumerable: true,
        configurable: true
    });
    RatingComponent.prototype.setRatingValue = function (value) {
        this.statValue = this.value = value;
    };
    RatingComponent.prototype.writeValue = function (obj) {
        this.setRatingValue(obj);
        this.changeDetectorRef.markForCheck();
    };
    RatingComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    RatingComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    RatingComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    RatingComponent.prototype.rate = function (value) {
        if (!this.disabled && this.value !== value) {
            this.onTouched();
            this.setRatingValue(value);
            this.valueChange.emit(this.value);
            this.onChange(this.value);
        }
    };
    RatingComponent.prototype.enter = function (value) {
        if (!this.disabled) {
            this.onTouched();
            this.statValue = value;
        }
    };
    RatingComponent.prototype.reset = function () {
        this.statValue = this.value;
    };
    RatingComponent.prototype.fillRatingRange = function (range) {
        return Array(range).fill(0).map(function (_, i) { return i + 1; });
    };
    var RatingComponent_1;
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], RatingComponent.prototype, "disabled", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], RatingComponent.prototype, "cssClass", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], RatingComponent.prototype, "icons", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Number),
        __metadata("design:paramtypes", [Number])
    ], RatingComponent.prototype, "max", null);
    RatingComponent = RatingComponent_1 = __decorate([
        Component({
            selector: 'hi-rating',
            template: "\n    <span aria-labelledby=\"default-rating\" class=\"rating {{cssClass}}\"><span class=\"slider\" (mouseleave)=\"reset()\" tabindex=\"0\" role=\"slider\" [attr.aria-valuemin]=\"0\" [attr.aria-valuemax]=\"ratingRanges.length\" [attr.aria-valuenow]=\"statValue\" [attr.aria-valuetext]=\"statValue\"><ng-container *ngFor=\"let item of ratingRanges\"><span class=\"sr-only\">({{statValue >= item? \"*\": \"\"}} )</span> <i (mouseenter)=\"enter(item)\" (click)=\"rate(item)\" class=\"{{ statValue < item ? icons.stateOff: icons.stateOn}}\" [attr.title]=\"item\"></i></ng-container></span></span>\n  ",
            changeDetection: ChangeDetectionStrategy.OnPush,
            exportAs: 'rating',
            providers: [{
                    provide: NG_VALUE_ACCESSOR,
                    useExisting: forwardRef(function () { return RatingComponent_1; }),
                    multi: true
                }]
        }),
        __metadata("design:paramtypes", [HiNGConfig, ChangeDetectorRef])
    ], RatingComponent);
    return RatingComponent;
}());
export { RatingComponent };
