export declare const DEFAULT_STORAGE_POOL_KEY = "storage:default";
export declare enum StorageType {
    memory = 0,
    sessionStorage = 1,
    localStorage = 2
}
export interface IStorage {
    getAll(pool: string): any;
    get(options: {
        pool?: string;
        key: string;
    }): Object;
    put(options: {
        pool?: string;
        key: string;
    }, value: Object): any;
    remove(options: {
        pool?: string;
        key?: string;
    }): any;
    removeAll(): any;
}
export declare class WebStorage implements IStorage {
    private webStorage;
    constructor(webStorage: Storage);
    getAll(pool: string): any;
    saveAll(pool: string, storage: any): void;
    get({ pool, key }: {
        pool?: string;
        key: string;
    }): Object;
    put({ pool, key }: {
        pool?: string;
        key: string;
    }, value: Object): any;
    remove({ pool, key }: {
        pool?: string;
        key?: string;
    }): void;
    removeAll(): void;
}
export declare class MemoryStorage implements IStorage {
    private storage;
    constructor();
    getAll(pool: string): any;
    get({ pool, key }: {
        pool?: string;
        key: string;
    }): Object;
    put({ pool, key }: {
        pool?: string;
        key: string;
    }, value: Object): void;
    remove({ pool, key }: {
        pool?: string;
        key?: string;
    }): void;
    removeAll(): void;
}
export declare class StorageService {
    private sessionStorage;
    private localStorage;
    private memoryStorage;
    private storages;
    private defaultStorageType;
    constructor();
    setDefaultStorageType(storageType: StorageType): void;
    getAll({ pool, storageType }: {
        pool: string;
        storageType?: StorageType;
    }): any;
    get({ pool, key, storageType }: {
        pool?: string;
        key: string;
        storageType?: StorageType;
    }): Object;
    put({ pool, key, storageType }: {
        pool?: string;
        key: string;
        storageType?: StorageType;
    }, value: Object): any;
    remove({ pool, key, storageType }: {
        pool?: string;
        key?: string;
        storageType?: StorageType;
    }): any;
    removeAll({ storageType }: {
        storageType?: StorageType;
    }): any;
    private setupStorages;
}
export declare class StorageFactory {
    private static storageService;
    static getStorageService(): StorageService;
}
export declare function Cacheable({ pool, key, storageType }?: {
    pool?: string;
    key?: string;
    storageType?: StorageType;
}): (target: any, name: string, methodInfo: any) => {
    value: (...args: any[]) => any;
};
export declare const STORAGE_PROVIDERS: Array<any>;
