var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component, Input, TemplateRef, ViewChild, ElementRef, Renderer2 } from '@angular/core';
import { TreeViewComponent } from './tree-view.component';
import { DraggableDirective } from '../draggable';
import { TreePanelComponent } from './tree-panel.component';
import { of } from 'rxjs/observable/of';
import 'rxjs/add/operator/map';
var TreeNodeComponent = (function () {
    function TreeNodeComponent(treeViewComponent, treePanelComponent, renderer) {
        var _this = this;
        this.treeViewComponent = treeViewComponent;
        this.treePanelComponent = treePanelComponent;
        this.renderer = renderer;
        this.checkable = false;
        this.lazyLoad = false;
        this.allowDraggable = false;
        this.allowMutipleSelected = false;
        this.isAcceptDrop = function ($event) {
            var dropData = JSON.parse($event.dataTransfer.getData(DraggableDirective.DRAGGABLE_DATA_KEY));
            if (!dropData.data.node) {
                return false;
            }
            if (_this.isNodeMyself(dropData)) {
                return false;
            }
            if (_this.isDescendant(dropData.data.node, _this.node)) {
                return false;
            }
            return true;
        };
    }
    TreeNodeComponent.prototype.getNodeExpendIcon = function () {
        if (this.isLeaf()) {
            return this.leafIcon;
        }
        return this.node.$expend ? this.expendIcon : this.collapseIcon;
    };
    TreeNodeComponent.prototype.onExpendedIconClick = function ($event) {
        var _this = this;
        $event.stopPropagation();
        if (this.isLeaf() || this.isLoading) {
            return;
        }
        var loadObservable = of(null);
        if (this.lazyLoad && !this.node.$loaded) {
            this.isLoading = true;
            loadObservable = this.loadChildren(this.node)
                .map(function (children) {
                _this.node.children = children || [];
                _this.node.$loaded = true;
            });
        }
        loadObservable.subscribe(function () {
            _this.isLoading = false;
            _this.node.$expend = !_this.node.$expend;
            _this.treeViewComponent.onNodeItemExpended(_this.node);
        }, function () {
            _this.isLoading = false;
        });
    };
    TreeNodeComponent.prototype.onNodeItemClick = function ($event) {
        $event.stopPropagation();
        this.node.$select = !this.node.$select;
        this.treeViewComponent.onNodeItemClicked(this.node);
    };
    TreeNodeComponent.prototype.onNodeItemDbClicked = function () {
        this.treeViewComponent.onNodeItemDbClicked(this.node);
    };
    TreeNodeComponent.prototype.nodeItemCheckedChange = function (checked) {
        this.changeChildrenChecked(this.node.children, checked);
        this.treePanelComponent.onNodeItemCheckedChanged(this.node);
    };
    TreeNodeComponent.prototype.onChildrenNodeCheckedChange = function (node) {
        this.node.$check = !this.node.children.some(function (item) { return !item.$check; });
        this.treePanelComponent.onNodeItemCheckedChanged(node);
    };
    TreeNodeComponent.prototype.isLeaf = function () {
        if (this.lazyLoad && !this.node.$loaded) {
            return false;
        }
        return !this.node.children || !this.node.children.length;
    };
    TreeNodeComponent.prototype.onDragEnter = function () {
        this.renderer.addClass(this.nodeItemContent.nativeElement, 'drop-node-enter');
    };
    TreeNodeComponent.prototype.onDragLeave = function () {
        var _this = this;
        setTimeout(function () {
            _this.renderer.removeClass(_this.nodeItemContent.nativeElement, 'drop-node-enter');
        });
    };
    TreeNodeComponent.prototype.onDropNodeItem = function ($event) {
        var _this = this;
        var dropData = JSON.parse($event.dataTransfer.getData(DraggableDirective.DRAGGABLE_DATA_KEY));
        if (dropData.data.node && !this.isNodeMyself(dropData)) {
            this.treeViewComponent.onNodeItemDroped({ target: this.node, data: dropData });
        }
        setTimeout(function () {
            _this.renderer.removeClass(_this.nodeItemContent.nativeElement, 'drop-node-enter');
        }, 0);
    };
    TreeNodeComponent.prototype.isNodeMyself = function (dropData) {
        return this.node[this.valueField] === dropData.data.node[this.valueField];
    };
    TreeNodeComponent.prototype.isDescendant = function (parent, target) {
        var _this = this;
        if (!parent) {
            return false;
        }
        return (parent.children || []).some(function (nodeItem) {
            return nodeItem[_this.valueField] === target[_this.valueField] || _this.isDescendant(nodeItem, target);
        });
    };
    TreeNodeComponent.prototype.changeChildrenChecked = function (nodes, checked) {
        var _this = this;
        if (!nodes) {
            return;
        }
        nodes.forEach(function (node) {
            node.$check = checked;
            _this.changeChildrenChecked(node.children, checked);
        });
    };
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TreeNodeComponent.prototype, "node", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TreeNodeComponent.prototype, "parentNode", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], TreeNodeComponent.prototype, "valueField", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], TreeNodeComponent.prototype, "textField", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], TreeNodeComponent.prototype, "nodeCssClass", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], TreeNodeComponent.prototype, "iconField", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TreeNodeComponent.prototype, "checkable", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TreeNodeComponent.prototype, "lazyLoad", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], TreeNodeComponent.prototype, "loadingIcon", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Function)
    ], TreeNodeComponent.prototype, "loadChildren", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TreeNodeComponent.prototype, "allowDraggable", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TreeNodeComponent.prototype, "allowMutipleSelected", void 0);
    __decorate([
        Input(),
        __metadata("design:type", TemplateRef)
    ], TreeNodeComponent.prototype, "nodeItemTemplate", void 0);
    __decorate([
        Input(),
        __metadata("design:type", TemplateRef)
    ], TreeNodeComponent.prototype, "nodeItemToolbarTemplate", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TreeNodeComponent.prototype, "leafIcon", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TreeNodeComponent.prototype, "expendIcon", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TreeNodeComponent.prototype, "collapseIcon", void 0);
    __decorate([
        ViewChild('nodeItemContent', { static: false }),
        __metadata("design:type", ElementRef)
    ], TreeNodeComponent.prototype, "nodeItemContent", void 0);
    TreeNodeComponent = __decorate([
        Component({
            selector: 'hi-tree-node,[reTreeNode]',
            template: "\n    <div class=\"node-item {{nodeCssClass}}\" reDraggable=\"re:node-item-drop\" [disabled]=\"!allowDraggable\" [dragData]=\"{node: node, parent: parentNode}\"><div class=\"node-item-panel\"><i class=\"node-expend-icon {{getNodeExpendIcon()}}\" (click)=\"onExpendedIconClick($event)\"></i><div class=\"node-item-content\" #nodeItemContent reDroppable=\"re:node-item-drop\" (onDrop)=\"onDropNodeItem($event)\" (onDragEnter)=\"onDragEnter()\" (onDragLeave)=\"onDragLeave()\" [acceptDrop]=\"isAcceptDrop\"><div class=\"node-check-box\"><hi-checkbox *ngIf=\"checkable\" [(ngModel)]=\"node.$check\" label=\"\" (valueChange)=\"nodeItemCheckedChange($event)\"></hi-checkbox></div><ng-container *ngIf=\"!nodeItemTemplate\"><span class=\"node-text\" [ngClass]=\"{'bg-primary': node.$select}\" (click)=\"onNodeItemClick($event)\" (dblclick)=\"onNodeItemDbClicked($event)\"><i *ngIf=\"iconField && node[iconField]\" class=\"node-icon {{node[iconField]}}\"></i> {{node[textField] || ''}} <span class=\"loading\" *ngIf=\"isLoading\"><i class=\"{{loadingIcon}}\"></i></span></span><div class=\"toolbar\"><ng-template *ngIf=\"nodeItemToolbarTemplate\" [ngTemplateOutlet]=\"nodeItemToolbarTemplate\" [ngTemplateOutletContext]=\"{$implicit: node, parentNode:parentNode, tree: this}\"></ng-template></div></ng-container><ng-container *ngIf=\"nodeItemTemplate\"><ng-template *ngIf=\"nodeItemTemplate\" [ngTemplateOutlet]=\"nodeItemTemplate\" [ngTemplateOutletContext]=\"{$implicit: node,  parentNode:parentNode, tree: this }\"></ng-template></ng-container></div></div><hi-tree-panel *ngIf=\"node.children && node.children.length && node.$expend\" class=\"children-view\" [treeData]=\"node.children\" [textField]=\"textField\" [valueField]=\"valueField\" [iconField]=\"iconField\" [checkable]=\"checkable\" [lazyLoad]=\"lazyLoad\" [loadingIcon]=\"loadingIcon\" [loadChildren]=\"loadChildren\" [allowDraggable]=\"allowDraggable && !isInDraggle\" [allowMutipleSelected]=\"allowMutipleSelected\" [parentNode]=\"node\" [nodeItemTemplate]=\"nodeItemTemplate\" [nodeItemToolbarTemplate]=\"nodeItemToolbarTemplate\" [leafIcon]=\"leafIcon\" [expendIcon]=\"expendIcon\" [collapseIcon]=\"collapseIcon\" (nodeItemCheckedChanged)=\"onChildrenNodeCheckedChange($event)\"></hi-tree-panel></div>\n  ",
            exportAs: 'treeNode'
        }),
        __metadata("design:paramtypes", [TreeViewComponent,
            TreePanelComponent,
            Renderer2])
    ], TreeNodeComponent);
    return TreeNodeComponent;
}());
export { TreeNodeComponent };
