var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component, Input, Output, EventEmitter, TemplateRef } from '@angular/core';
import { HiNGConfig } from '../hi.config';
import { TreeViewService } from './tree-view.service';
var TreeViewComponent = (function () {
    function TreeViewComponent(hiNGConfig, treeViewService) {
        this.treeViewService = treeViewService;
        this.checkable = false;
        this.lazyLoad = false;
        this.allowDraggable = false;
        this.allowMutipleSelected = false;
        this.nodeItemClicked = new EventEmitter();
        this.nodeItemDroped = new EventEmitter();
        this.nodeItemDbClicked = new EventEmitter();
        this.nodeItemCheckedChanged = new EventEmitter();
        this.nodeItemExpended = new EventEmitter();
        this.valueField = hiNGConfig.treeView.valueField;
        this.textField = hiNGConfig.treeView.textField;
        this.leafIcon = hiNGConfig.treeView.leafIcon;
        this.expendIcon = hiNGConfig.treeView.expendIcon;
        this.collapseIcon = hiNGConfig.treeView.collapseIcon;
        this.loadingIcon = hiNGConfig.treeView.loadingIcon;
    }
    TreeViewComponent.prototype.onNodeItemExpended = function (node) {
        this.nodeItemExpended.emit(node);
    };
    TreeViewComponent.prototype.onNodeItemClicked = function (node) {
        if (!this.allowMutipleSelected) {
            if (this.selectNode && this.selectNode !== node) {
                this.selectNode.$select = false;
            }
            this.selectNode = node;
        }
        this.nodeItemClicked.emit(node);
    };
    TreeViewComponent.prototype.onNodeItemDbClicked = function (node) {
        this.nodeItemDbClicked.emit(node);
    };
    TreeViewComponent.prototype.onNodeItemCheckedChanged = function (node) {
        this.nodeItemCheckedChanged.emit(node);
    };
    TreeViewComponent.prototype.onNodeItemDroped = function ($event) {
        var _this = this;
        var target = $event.target;
        if (!target) {
            return;
        }
        var source = $event.data.data.node;
        var sourceParent = $event.data.data.parent;
        if (sourceParent && sourceParent[this.valueField]) {
            var matchNode = this.getTreeNodeByValue(sourceParent[this.valueField]);
            if (matchNode) {
                matchNode.node.children = matchNode.node.children
                    .filter(function (nodeItem) { return nodeItem[_this.valueField] !== source[_this.valueField]; });
            }
        }
        else {
            if (this.treeData.length < 2) {
                return;
            }
            this.treeData = this.treeData.filter(function (nodeItem) { return nodeItem[_this.valueField] !== source[_this.valueField]; });
        }
        target.children = target.children || [];
        source.$select = false;
        target.children.push(source);
        this.nodeItemDroped.emit($event);
    };
    TreeViewComponent.prototype.getSelectNodes = function () {
        return this.treeViewService.getSelectNodes(this.treeData);
    };
    TreeViewComponent.prototype.getCheckedNodes = function () {
        return this.treeViewService.getCheckedNodes(this.treeData);
    };
    TreeViewComponent.prototype.getMatchedItems = function (match) {
        return this.treeViewService.getMatchedItems(this.treeData, match);
    };
    TreeViewComponent.prototype.getFirstMatchedItem = function (match) {
        return this.treeViewService.getFirstMatchedItem(this.treeData, match);
    };
    TreeViewComponent.prototype.getTreeNodeByValue = function (value) {
        return this.treeViewService.getTreeNodeByValue(this.treeData, this.valueField, value);
    };
    TreeViewComponent.prototype.expendAllNodes = function () {
        return this.treeViewService.expendAllNodes(this.treeData);
    };
    TreeViewComponent.prototype.collapseAllNodes = function () {
        return this.treeViewService.collapseAllNodes(this.treeData);
    };
    TreeViewComponent.prototype.expendNodesByLevel = function (level) {
        this.treeViewService.expendNodesByLevel(this.treeData, level);
    };
    TreeViewComponent.prototype.expendNodesByValue = function (value) {
        this.treeViewService.expendNodesByValue(this.treeData, this.valueField, value);
    };
    TreeViewComponent.prototype.getNodePathByValue = function (value) {
        return this.treeViewService.getNodePathByValue(this.treeData, this.valueField, value);
    };
    TreeViewComponent.prototype.checkAllNodes = function () {
        return this.treeViewService.checkAllNodes(this.treeData);
    };
    TreeViewComponent.prototype.unCheckAllNodes = function () {
        return this.treeViewService.unCheckAllNodes(this.treeData);
    };
    TreeViewComponent.prototype.appendNodes = function (parentId, nodes) {
        return this.treeViewService.appendNodes(this.treeData, this.valueField, parentId, nodes);
    };
    TreeViewComponent.prototype.removeNode = function (value) {
        return this.treeViewService.removeNode(this.treeData, this.valueField, value);
    };
    __decorate([
        Input(),
        __metadata("design:type", Array)
    ], TreeViewComponent.prototype, "treeData", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TreeViewComponent.prototype, "valueField", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TreeViewComponent.prototype, "textField", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], TreeViewComponent.prototype, "cssClass", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], TreeViewComponent.prototype, "nodeCssClass", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], TreeViewComponent.prototype, "iconField", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TreeViewComponent.prototype, "checkable", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TreeViewComponent.prototype, "lazyLoad", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], TreeViewComponent.prototype, "loadingIcon", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Function)
    ], TreeViewComponent.prototype, "loadChildren", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TreeViewComponent.prototype, "allowDraggable", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TreeViewComponent.prototype, "allowMutipleSelected", void 0);
    __decorate([
        Input(),
        __metadata("design:type", TemplateRef)
    ], TreeViewComponent.prototype, "nodeItemTemplate", void 0);
    __decorate([
        Input(),
        __metadata("design:type", TemplateRef)
    ], TreeViewComponent.prototype, "nodeItemToolbarTemplate", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TreeViewComponent.prototype, "leafIcon", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TreeViewComponent.prototype, "expendIcon", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TreeViewComponent.prototype, "collapseIcon", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], TreeViewComponent.prototype, "nodeItemClicked", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], TreeViewComponent.prototype, "nodeItemDroped", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], TreeViewComponent.prototype, "nodeItemDbClicked", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], TreeViewComponent.prototype, "nodeItemCheckedChanged", void 0);
    __decorate([
        Output(),
        __metadata("design:type", Object)
    ], TreeViewComponent.prototype, "nodeItemExpended", void 0);
    TreeViewComponent = __decorate([
        Component({
            selector: 'hi-tree-view',
            template: "\n    <hi-tree-panel class=\"{{cssClass}}\" [treeData]=\"treeData\" [textField]=\"textField\" [valueField]=\"valueField\" [nodeCssClass]=\"nodeCssClass\" [iconField]=\"iconField\" [checkable]=\"checkable\" [lazyLoad]=\"lazyLoad\" [loadingIcon]=\"loadingIcon\" [loadChildren]=\"loadChildren\" [allowDraggable]=\"allowDraggable\" [nodeItemTemplate]=\"nodeItemTemplate\" [nodeItemToolbarTemplate]=\"nodeItemToolbarTemplate\" [leafIcon]=\"leafIcon\" [expendIcon]=\"expendIcon\" [collapseIcon]=\"collapseIcon\" [allowMutipleSelected]=\"allowMutipleSelected\" (nodeItemCheckedChanged)=\"onNodeItemCheckedChanged($event)\"></hi-tree-panel>\n  ",
            exportAs: 'treeView'
        }),
        __metadata("design:paramtypes", [HiNGConfig, TreeViewService])
    ], TreeViewComponent);
    return TreeViewComponent;
}());
export { TreeViewComponent };
